/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "XPathEvaluator.h"
#include "XPathExpression.h"
#include "XPathNSResolver.h"
#include "XPathResult.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomXPathEvaluator.h"
#include "gdom/GdomXPathExpression.h"
#include "gdom/GdomXPathNSResolver.h"
#include "gdom/GdomXPathResult.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomXPathEvaluator.h"
#include "gdom/PrivateGdomXPathExpression.h"
#include "gdom/PrivateGdomXPathNSResolver.h"
#include "gdom/PrivateGdomXPathResult.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomXPathEvaluator.h"
#include "gdom/PrivateIntGdomXPathExpression.h"
#include "gdom/PrivateIntGdomXPathNSResolver.h"
#include "gdom/PrivateIntGdomXPathResult.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::XPathEvaluator* obj)
{
    return WebCore::cacheDOMObject<WebCore::XPathEvaluator, GDOMXPathEvaluator>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMXPathEvaluator::GDOMXPathEvaluator(PassRefPtr<WebCore::XPathEvaluator> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMXPathEvaluator::~GDOMXPathEvaluator()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

GdomXPathExpression *
gdom_x_path_evaluator_create_expression (GdomXPathEvaluator *thiz, gchar * expression, GdomXPathNSResolver * resolver)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XPathEvaluator * item = (gdom_x_path_evaluator_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (expression, 0);
    g_return_val_if_fail (resolver, 0);
    WebCore::String _g_expression = WebCore::String::fromUTF8(expression);
    WebCore::XPathNSResolver * _g_resolver = (gdom_x_path_ns_resolver_get_private(resolver));
    g_return_val_if_fail (_g_resolver, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::XPathExpression> g_res = WTF::getPtr(item->createExpression(_g_expression, _g_resolver, ec));
    GdomXPathExpression * res = static_cast<GdomXPathExpression *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomXPathNSResolver *
gdom_x_path_evaluator_create_ns_resolver (GdomXPathEvaluator *thiz, GdomNode * node_resolver)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XPathEvaluator * item = (gdom_x_path_evaluator_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (node_resolver, 0);
    WebCore::Node * _g_node_resolver = (gdom_node_get_private(node_resolver));
    g_return_val_if_fail (_g_node_resolver, 0);
    PassRefPtr<WebCore::XPathNSResolver> g_res = WTF::getPtr(item->createNSResolver(_g_node_resolver));
    GdomXPathNSResolver * res = static_cast<GdomXPathNSResolver *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomXPathResult *
gdom_x_path_evaluator_evaluate (GdomXPathEvaluator *thiz, gchar * expression, GdomNode * context_node, GdomXPathNSResolver * resolver, gushort type, GdomXPathResult * in_result)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XPathEvaluator * item = (gdom_x_path_evaluator_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (expression, 0);
    g_return_val_if_fail (context_node, 0);
    g_return_val_if_fail (resolver, 0);
    g_return_val_if_fail (in_result, 0);
    WebCore::String _g_expression = WebCore::String::fromUTF8(expression);
    WebCore::Node * _g_context_node = (gdom_node_get_private(context_node));
    g_return_val_if_fail (_g_context_node, 0);
    WebCore::XPathNSResolver * _g_resolver = (gdom_x_path_ns_resolver_get_private(resolver));
    g_return_val_if_fail (_g_resolver, 0);
    WebCore::XPathResult * _g_in_result = (gdom_x_path_result_get_private(in_result));
    g_return_val_if_fail (_g_in_result, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::XPathResult> g_res = WTF::getPtr(item->evaluate(_g_expression, _g_context_node, _g_resolver, type, _g_in_result, ec));
    GdomXPathResult * res = static_cast<GdomXPathResult *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "XPathEvaluator.h"
#include <gdom/GdomXPathEvaluator.h>
#include <gdom/PrivateGdomXPathEvaluator.h>
#include <gdom/PrivateIntGdomXPathEvaluator.h>
#include <gdom/GdomXPathEvaluatorprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomXPathEvaluator, gdom_x_path_evaluator, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomXPathEvaluatorPrivate {
    class WebCore::XPathEvaluator *item;
};

#define GDOM_X_PATH_EVALUATOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_X_PATH_EVALUATOR, GdomXPathEvaluatorPrivate))
*/
G_END_DECLS
WebCore::XPathEvaluator *
gdom_x_path_evaluator_get_private(GdomXPathEvaluator *request)
{
    GDOMXPathEvaluator *res;
    res = static_cast<GDOMXPathEvaluator*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::XPathEvaluator*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
};


static void gdom_x_path_evaluator_finalize(GObject* object)
{
    /*GdomXPathEvaluator* request = GDOM_X_PATH_EVALUATOR(object);*/
    /*GdomXPathEvaluatorPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_x_path_evaluator_parent_class)->finalize(object);
}

static void gdom_x_path_evaluator__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomXPathEvaluator *self = GDOM_X_PATH_EVALUATOR(object);
    WebCore::XPathEvaluator * gself = (gdom_x_path_evaluator_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_x_path_evaluator__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomXPathEvaluator *self = GDOM_X_PATH_EVALUATOR(object);
    WebCore::XPathEvaluator * gself = (gdom_x_path_evaluator_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_x_path_evaluator_class_init(GdomXPathEvaluatorClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_x_path_evaluator_finalize;
    g_klass->set_property = gdom_x_path_evaluator__set_property_fn;
    g_klass->get_property = gdom_x_path_evaluator__get_property_fn;





    /*g_type_class_remove_private(requestClass, sizeof(GdomXPathEvaluatorPrivate));*/
}

static void gdom_x_path_evaluator_init(GdomXPathEvaluator* request)
{
    /*GdomXPathEvaluatorPrivate* priv = GDOM_X_PATH_EVALUATOR_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

